/**
 * Fractal Binding Factory.
 * Copyright (C) 2007-2008 INRIA, SARDES

 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: Valerio.Schiavoni@gmail.com
 * 
 * Author: Valerio Schiavoni
 * 
 */
package org.objectweb.fractal.bf.connectors.ws;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.logging.Logger;

import org.apache.cxf.aegis.databinding.AegisDatabinding;
import org.apache.cxf.frontend.ClientProxyFactoryBean;
import org.objectweb.fractal.bf.AbstractRemoteServiceCaller;
import org.objectweb.fractal.bf.StubGenerationException;

/**
 * Handle calls to remote web-services.
 */
public class WebServiceCaller extends AbstractRemoteServiceCaller {

	Logger log = Logger.getLogger(WebServiceCaller.class.getCanonicalName());

	public WebServiceCaller(WsBindHints bindHints, Class<?> clazz) {

		if (bindHints == null) {
			throw new StubGenerationException("WsBindHInts can't be null");
		}

		log.finest("Class name for client reference: " + clazz.getName());

		/*
		 * this if block is useful only for pre-generated wrapper classes (via
		 * wsdl2java tools). It should be possible to split this plugin directly
		 * into two distict plugin, one of which would avoid the dependency to
		 * CXF.
		 */
		if (clazz.getName().endsWith("Soap")) {
			// this block of code is for pre-generated stubs
			String canonicalName = clazz.getCanonicalName();
			String simpleName = clazz.getSimpleName();
			log.finest("Reference class simple name: " + canonicalName);
			String soapEnabledClassName = clazz.getCanonicalName().substring(0,
					canonicalName.length() - 4);
			log.finest("soap enabled class name: " + soapEnabledClassName);
			try {
				Class<?> factoryClass = Class.forName(soapEnabledClassName);
				Object factoryInstance = factoryClass.newInstance();
				// XXX hard-coding the name-scheme used by wsdl2java and similar
				// tools
				Method factoryMethod = factoryInstance.getClass().getMethod(
						"get" + simpleName, new Class[] {});
				this.reference = factoryMethod.invoke(factoryInstance,
						new Object[] {});
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
				log.warning("Could not find class :" + soapEnabledClassName
						+ " in the classpath");
			} catch (InstantiationException e) {
				throw new StubGenerationException(e);
			} catch (IllegalAccessException e) {
				throw new StubGenerationException(e);
			} catch (SecurityException e) {
				throw new StubGenerationException(e);
			} catch (NoSuchMethodException e) {
				throw new StubGenerationException(e);
			} catch (IllegalArgumentException e) {
				throw new StubGenerationException(e);
			} catch (InvocationTargetException e) {
				throw new StubGenerationException(e);
			}

		} else {
			// use dynamic generation with cxf proxy factories
			final ClientProxyFactoryBean clientProxyfactoryBean = new ClientProxyFactoryBean();
			clientProxyfactoryBean.setServiceClass(clazz);
			String address = bindHints.getAddress();
			log.finer("Address: " + address);
			clientProxyfactoryBean.setAddress(address);
			final AegisDatabinding aegisDatabinding = new AegisDatabinding();

			// with cxf 2.0.x
			HashMap<String, Object> props = new HashMap<String, Object>();
			props.put(AegisDatabinding.WRITE_XSI_TYPE_KEY, true);
			clientProxyfactoryBean.setProperties(props);
			// with cxf 2.1
			// AegisContext aegisCtx = new AegisContext();
			// aegisCtx.setReadXsiTypes(true);
			// aegisDatabinding.setAegisContext(aegisCtx);
			clientProxyfactoryBean.setDataBinding(aegisDatabinding);
			this.reference = clientProxyfactoryBean.create();

			// final Client cxfclient = ClientProxy.getClient(client);
			// HTTPConduit conduit = (HTTPConduit) cxfclient.getConduit();
			// HTTPClientPolicy policy = conduit.getClient();
			//policy.setReceiveTimeout(Integer.valueOf(bindHints.getTimeout()));
		}

	}

}
