/**
 * Fractal Binding Factory.
 * Copyright (C) 2007-2008 INRIA, SARDES

 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: Valerio.Schiavoni@gmail.com
 * 
 * Author: Valerio Schiavoni
 * 
 */
package org.objectweb.fractal.bf.connectors.ws;

import java.util.HashMap;
import java.util.logging.Logger;

import org.apache.cxf.aegis.databinding.AegisDatabinding;
import org.apache.cxf.endpoint.Server;
import org.apache.cxf.frontend.ServerFactoryBean;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.control.LifeCycleController;
import org.objectweb.fractal.bf.AbstractSkeleton;

/**
 * This class uses CXF SimpleFrontEnd (see {@link here
 * http://incubator.apache.org
 * /cxf/javadoc/latest/org/apache/cxf/frontend/ServerFactoryBean.html } ) to
 * create a web-service from a base java class.
 * 
 * @author Valerio Schiavoni <valerio.schiavoni@gmail.com>
 */

public class WsServerBridge extends AbstractSkeleton implements
		WsConnectorAttributes, LifeCycleController {

	Logger log = Logger.getLogger(WsServerBridge.class.getCanonicalName());

	/**
	 * the fcState of this component.
	 */
	String fcState;

	ServerFactoryBean wsServerFactoryBean;

	/**
	 * A reference to the server bean, once it's been started.
	 */
	private Server server;

	private String itfName;

	public WsServerBridge() {
		wsServerFactoryBean = new ServerFactoryBean();
		final AegisDatabinding aegisDatabinding = new AegisDatabinding();
		HashMap<String, Object> props = new HashMap<String, Object>();
		props.put(AegisDatabinding.WRITE_XSI_TYPE_KEY, new Boolean(true));
		wsServerFactoryBean.setProperties(props);
		wsServerFactoryBean.getServiceFactory()
				.setDataBinding(aegisDatabinding);

	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#getFcState()
	 */
	public String getFcState() {
		return this.fcState;
	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
	 */
	public void startFc() throws IllegalLifeCycleException {
		try {

			final Object delegateItf = delegate.getFcInterface(itfName);
			wsServerFactoryBean.setServiceBean(delegateItf);
			this.server = wsServerFactoryBean.create();
			this.fcState = LifeCycleController.STARTED;

		} catch (NoSuchInterfaceException e) {
			IllegalLifeCycleException ilce = new IllegalLifeCycleException(
					"Cannot start the ws skeleton");
			ilce.initCause(e);
			throw ilce;
		}

	}

	/**
	 * @see org.objectweb.fractal.api.control.LifeCycleController#stopFc()
	 */
	public void stopFc() throws IllegalLifeCycleException {
		this.server.stop();
		this.fcState = LifeCycleController.STOPPED;

	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsConnectorAttributes#setAddress(java.lang.String)
	 */
	public void setAddress(String address) {
		wsServerFactoryBean.setAddress(address);
	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsConnectorAttributes#setServiceClass(java.lang.Class)
	 */
	public void setServiceClass(Class<?> serviceClass) {
		wsServerFactoryBean.setServiceClass(serviceClass);
	}

	/**
	 * @see org.objectweb.fractal.bf.connectors.ws.WsConnectorAttributes#setInterfaceName(java.lang.String)
	 */
	public void setInterfaceName(String itfName) {
		this.itfName = itfName;

	}
}
